<?php

namespace App\Http\Controllers\Api;

use Carbon\Carbon;
use Illuminate\Http\Request;

use App\Models\PriceCategory;
use App\Models\ProductCategory;
use App\Repositories\RoleRepository;
use App\Repositories\SliderRepository;
use App\Repositories\ContactRepository;
use App\Repositories\InviteCodeRepository;
use App\Repositories\ClientPriceRepository;
use App\Repositories\PriceCategoryRepository;

use App\Repositories\StockOperationRepository;
use App\Http\Controllers\Api\AppBaseController;
use App\Repositories\ReductionIntervalRepository;
use Illuminate\Support\Facades\Cache;

class RootAppController extends AppBaseController
{

    protected $roleRepository;
    protected $clientPriceRepository;
    protected $reductionIntervalRepository;
    protected $inviteCodeRepository;
    protected $contactRepository;
    protected $priceCategoryRepository;
    protected $sliderRepository;

    protected $stockOperationRepository;

    public function __construct(
        RoleRepository $roleRepository,
        ClientPriceRepository $clientPriceRepository,
        ReductionIntervalRepository $reductionIntervalRepository,
        StockOperationRepository $stockOperationRepository,
        InviteCodeRepository $inviteCodeRepository,
        ContactRepository $contactRepository,
        PriceCategoryRepository $priceCategoryRepository,
        SliderRepository $sliderRepository
    ) {
        $this->roleRepository = $roleRepository;
        $this->clientPriceRepository = $clientPriceRepository;
        $this->reductionIntervalRepository = $reductionIntervalRepository;
        $this->inviteCodeRepository = $inviteCodeRepository;
        $this->stockOperationRepository = $stockOperationRepository;
        $this->contactRepository = $contactRepository;
        $this->priceCategoryRepository = $priceCategoryRepository;
        $this->sliderRepository = $sliderRepository;
    }

    public function getSliderByTag(Request $request) {

        $sliders = $this->sliderRepository->all([
            "tag" => $request->tag
        ]);

        return response()->json([
            "sliders" => $sliders
        ]);

    }
    
    public function getDeleveryFees(Request $request) {
        
        $prices = \App\Models\Price::whereHas('tags', function($query) {
            $query->where('tag', 'delivery_fees');
        })->get();
        
        $response = [];
        $enableCashDelevry = [
            'LIVRAISON CASABLANCA 20 DHS', 
            'MOHMADIA / BOUSKOURA / BERRCHID 39 DHS', 
            'RABAT KENITRA SALÉ ET RÉGIONS', 
            'LIVRAISON RAHMA /DEROUA/TITT MELIL ...30 DHS',
            'RABAT//KENITRA//SALE//TEMARA//ET REGIONS 35 DHS'
        ];
        
        foreach($prices as $price) {
            $detail = $price->products[0]->detailKeys[0];
            
            $response[] = (object) [
                'id' => $detail->id,
                'label' => $detail->value,
                'price' => $price->price > $price->promoPrice ? $price->promoPrice : $price->price,
                'price_id' => $price->id,
                'cashDelevry' => in_array($detail->value, $enableCashDelevry),
                'bankTransfer' => true,
            ];
        }
        
        return response()->json([
            'data' => $response
        ]);
    }

    public function getRootCategories(Request $request) {
        $categories = ProductCategory::select('label', 'tag', 'id', 'file_path')
        ->where('isDefault', 1)
        ->whereNull('parent_id')
        ->where('isHidden', false)
        ->with(['childrens' => function ($query) {
            $query
                ->select('label', 'tag', 'id', 'file_path')
                ->where('isHidden', false);
        }])
        ->orderBy('order', 'asc')
        ->with('parent')
        ->get();

        return response()->json([
            "categories" => $categories
        ]);
    }

    public function getAllCategory(Request $request) {

        $categories = ProductCategory::select('label', 'tag', 'id', 'parent_id', 'file_path')
        ->where('isHidden', false)
        ->orderBy('order', 'asc')
        ->get();

        return response()->json([
            "categories" => $categories
        ]);
    }

    public function getAllCategoryTest(Request $request) {

        // $categories = ProductCategory::select('label', 'tag', 'id', 'parent_id', 'file_path')
        // ->where('isHidden', false)
        // ->orderBy('order', 'asc')
        // ->get();
        $categories = Cache::remember('product_categories', now()->addHour(), function () {
            return ProductCategory::select('label', 'tag', 'id', 'parent_id', 'file_path')
                ->where('isHidden', false)
                ->orderBy('order', 'asc')
                ->limit(1)
                ->get();
        });

        return response()->json([
            "categories" => $categories
        ]);
    }

    public function addContact(Request $request){

        $contactModel = $this->contactRepository->create([
            "name" => $request->name,
            "email" => $request->email,
            "phone" => $request->phone,
            "message" => $request->message
        ]);

        return response()->json([
            'status' => 'ok'
        ], 200);

    }

    public function getInitData(Request $request){
        $constants = setting('constants');
        return response()->json([
            'constants' => $constants
        ], 200);
    }

    public function getReductions(Request $request){

        $payloadDate = Carbon::parse($request->date);
        if (!$payloadDate) $payloadDate = Carbon::parse($request->header('User-Date'));

        $intervalReduction = $this->reductionIntervalRepository->reductionByHour($payloadDate);

        return response()->json([
            'reduction' => $intervalReduction
        ], 200);
    }

    public function getStockStatus(Request $request){

        $stockStatus = $this->stockOperationRepository->stockStatus();
        return response()->json([
            'stockStatus' => $stockStatus
        ], 200);

    }

    public function getProductStockStatus(Request $request){
        // \Log::debug($request->all());
        $ids = explode(",", $request->ids);
        $stockStatus = $this->stockOperationRepository->stockStatus($ids);
        return response()->json([
            'stockStatus' => $stockStatus
        ], 200);
    }

    public function getCommandByHash(Request $request){

        $inviteCode = $this->inviteCodeRepository->allQuery([
            "hash" => $request->hash
        ])->first();

        $command = $inviteCode->command()->with("address.geographicPoint")->first();

        return response()->json([
            'command' => $command
        ], 200);

    }

    public function getAuthUser(Request $request) {

        $user = \JWTAuth::toUser();
        $entreprise = $user->entreprise;
        
        $client = $user->client;

        $likedPrices = $this->clientPriceRepository->all([
            "clientId" => $client->id
        ])->map(function ($item) { return $item->priceId; });

        $userObj = $user->toArray();
        $userObj["client"] = $client;
        $userObj["likedPrices"] = $likedPrices;

        return response()->json([
            'user' => $userObj,
            'entreprise' => $entreprise
        ], 200);

    }

    public function getDeliveryAuthUser(Request $request) {

        $user = \JWTAuth::toUser();

        $role = $this->roleRepository->firstOrCreate("livreur");

        if($user->role_id == $role->id) $userObj = $user->toArray();
        else {
            return response()->json([
                'message' => "Unauthorized to access the delivrey page"
            ], 401);
        }

        return response()->json([
            'user' => $userObj
        ], 200);
    }

    public function favoritProduct($priceId) {

        $user = \JWTAuth::toUser();

        $client = $user->client;

        $clientPrice = $this->clientPriceRepository->create([
            "priceId" => $priceId,
            "clientId" => $client->id
        ]);

        return response()->json([
            'clientPrice' => $clientPrice
        ], 200);

    }

    public function deleteAccount()
    {
        $user = \JWTAuth::toUser();
        $user->client->delete();
        $user->delete();
        return response()->json([true]);
    }
}
